import 'package:cl_datahub_common/common.dart';

import 'status_code_exception.dart';
import 'package:http/http.dart';
import 'package:boost/boost.dart';

class ApiResponse<TResult> {
  final Uri? requestUrl;
  final int statusCode;
  final TResult? _data;

  bool get hasData => _data != null;
  TResult get data =>
      _data ?? (throw ApiException('Response does not contain data.'));

  ApiResponse(Response response, this._data)
      : requestUrl = response.request?.url,
        statusCode = response.statusCode;

  void throwOnError() {
    if (statusCode >= 400) {
      throw StatusCodeException.fromResponse(this);
    }

    if (!isSubtypeOf<void, TResult>() && _data == null) {
      throw ApiException('Response does not contain data.');
    }
  }
}
